﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Telerik.Windows.Controls;

namespace BaseApplicationProject
{
    public class MainViewModel : ViewModelBase
    {
        private bool isBusy;
        private string busyContent;
        private ObservableCollection<DataItem> dataItems;
        private Random random = new Random();

        public MainViewModel()
        {
        }

        public ObservableCollection<DataItem> DataItems
        {
            get { return this.dataItems; }
            set
            {
                if (this.dataItems != value)
                {
                    this.dataItems = value;
                    this.OnPropertyChanged(() => this.DataItems);
                }
            }
        }

        public bool IsBusy
        {
            get { return this.isBusy; }
            set
            {
                if (this.isBusy != value)
                {
                    this.isBusy = value;
                    this.OnPropertyChanged(() => this.IsBusy);

                    if (this.isBusy)
                    {
                        var backgroundWorker = new BackgroundWorker();
                        backgroundWorker.DoWork += this.OnBackgroundWorkerDoWork;
                        backgroundWorker.RunWorkerCompleted += OnBackgroundWorkerRunWorkerCompleted;
                        backgroundWorker.RunWorkerAsync();
                    }
                }
            }
        }

        public string BusyContent
        {
            get { return this.busyContent; }
            set
            {
                if (this.busyContent != value)
                {
                    this.busyContent = value;
                    this.OnPropertyChanged(() => this.BusyContent);
                }
            }
        }

        private void OnBackgroundWorkerRunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            var backgroundWorker = sender as BackgroundWorker;
            backgroundWorker.DoWork -= this.OnBackgroundWorkerDoWork;
            backgroundWorker.RunWorkerCompleted -= OnBackgroundWorkerRunWorkerCompleted;

            InvokeOnUIThread(() =>
            {
                this.IsBusy = false;
                this.DataItems = new ObservableCollection<DataItem>((IEnumerable<DataItem>)e.Result);
            });
        }

        private void OnBackgroundWorkerDoWork(object sender, DoWorkEventArgs e)
        {
            e.Result = this.GenerateDataItems().ToList();
        }

        private IEnumerable<DataItem> GenerateDataItems()
        {
            Random r = new Random();
            for (int i = 0; i < 1000; i++)
            {

                    yield return new DataItem
                    {
                        Value = r.Next(10, 30)
                    };

                    System.Threading.Thread.Sleep(1);
                }
            }
        }

    public class DataItem
    {
        public string Category { get; set; }
        public double Value { get; set; }
    }
    }



